@@ -11,7 +11,7 @@
  *                                                                  *
  ********************************************************************
 
- last mod: $Id: callbacks.c,v 1.7 2002/07/06 04:27:19 msmith Exp $
+ last mod: $Id: callbacks.c,v 1.9 2003/09/05 19:25:30 volsung Exp $
 
  ********************************************************************/
 
@@ -61,7 +61,8 @@ void audio_reopen_action (buf_t *buf, void *arg)
 				     current->options);
     else
       current->device = ao_open_file(current->driver_id, current->filename,
-				     0, &format, current->options);
+				     1 /*overwrite*/, &format, 
+				     current->options);
     
     /* Report errors */
     if (current->device == NULL) {
@@ -351,5 +352,6 @@ void decoder_buffered_metadata_callback (void *arg, int verbosity,
     }
   }
 
+  sm_arg->verbosity = verbosity;
   buffer_append_action_at_end(buf, &status_message_action, sm_arg);
 }
