@@ -14,7 +14,7 @@
  *                                                                  *
  ********************************************************************
 
- last mod: $Id: ogg123.c,v 1.65 2002/07/06 19:12:18 volsung Exp $
+ last mod: $Id: ogg123.c,v 1.70 2003/09/02 19:37:05 volsung Exp $
 
  ********************************************************************/
 
@@ -140,6 +140,7 @@ void options_init (ogg123_options_t *opts)
   opts->nth = 1;
   opts->ntimes = 1;
   opts->seekpos = 0.0;
+  opts->endpos = -1.0; /* Mark as unset */
   opts->buffer_size = 128 * 1024;
   opts->prebuffer = 0.0f;
   opts->input_buffer_size = 64 * 1024;
@@ -180,7 +181,10 @@ void select_stats (stat_format_t *stats, ogg123_options_t *opts,
     stats[7].enabled = 0;
   }
     
-  /* Put logic here to decide if this stream needs a total time display */
+  /* Assume we need total time display, and let display_statistics()
+     determine at what point it should be turned off during playback */
+  stats[2].enabled = 1;  /* Remaining playback time */
+  stats[3].enabled = 1;  /* Total playback time */
 }
 
 
@@ -250,6 +254,18 @@ void display_statistics_quick (stat_format_t *stat_format,
     print_statistics_action(NULL, pstats_arg);
 }
 
+double current_time (decoder_t *decoder)
+{
+  decoder_stats_t *stats;
+  double ret;
+
+  stats = decoder->format->statistics(decoder);
+  ret = stats->current_time;
+
+  free(stats);
+
+  return ret;
+}
 
 void print_audio_devices_info(audio_device_t *d)
 {
@@ -416,11 +432,11 @@ void play (char *source_string)
   /* Select appropriate callbacks */
   if (audio_buffer != NULL) {
     decoder_callbacks.printf_error = &decoder_buffered_error_callback;
-    decoder_callbacks.printf_metadata = &decoder_buffered_error_callback;
+    decoder_callbacks.printf_metadata = &decoder_buffered_metadata_callback;
     decoder_callbacks_arg = audio_buffer;
   } else {
     decoder_callbacks.printf_error = &decoder_error_callback;
-    decoder_callbacks.printf_metadata = &decoder_error_callback;
+    decoder_callbacks.printf_metadata = &decoder_metadata_callback;
     decoder_callbacks_arg = NULL;
   }
 
@@ -470,8 +486,12 @@ void play (char *source_string)
 
   /* Skip over audio */
   if (options.seekpos > 0.0) {
-    if (!format->seek(decoder, options.seekpos, DECODER_SEEK_START))
+    if (!format->seek(decoder, options.seekpos, DECODER_SEEK_START)) {
       status_error(_("Could not skip %f seconds of audio."), options.seekpos);
+      if (audio_buffer != NULL)
+	buffer_thread_kill(audio_buffer);
+      return;
+    }
   }
 
   /* Main loop:  Iterates over all of the logical bitstreams in the file */
@@ -540,6 +560,11 @@ void play (char *source_string)
       } else
 	next_status -= ret;
 
+      if (options.endpos > 0.0 && options.endpos <= current_time(decoder)) {
+	eof = eos = 1;
+	break;
+      }
+
 
       /* Write audio data block to output, skipping or repeating chunks
 	 as needed */
@@ -554,7 +579,8 @@ void play (char *source_string)
 	  nthc = options.nth - 1;
 	}
 	
-      } while (++ntimesc < options.ntimes);
+      } while (!sig_request.exit && !sig_request.skipfile &&
+	       ++ntimesc < options.ntimes);
 
       ntimesc = 0;
             
