@@ -3,7 +3,7 @@
  ** This program is distributed under the GNU General Public License, version 2.
  ** A copy of this license is included with this source.
  **
- ** Copyright 2000-2002, Michael Smith <msmith@labyrinth.net.au>
+ ** Copyright 2000-2002, Michael Smith <msmith@xiph.org>
  **
  ** AIFF/AIFC support from OggSquish, (c) 1994-1996 Monty <xiphmont@xiph.org>
  **/
@@ -19,6 +19,10 @@
 #include "i18n.h"
 #include "resample.h"
 
+#ifdef HAVE_LIBFLAC
+#include "flac.h"
+#endif
+
 #define WAV_HEADER_SIZE 44
 
 /* Macros to read header data */
@@ -38,6 +42,10 @@
 input_format formats[] = {
 	{wav_id, 12, wav_open, wav_close, "wav", N_("WAV file reader")},
 	{aiff_id, 12, aiff_open, wav_close, "aiff", N_("AIFF/AIFC file reader")},
+#ifdef HAVE_LIBFLAC
+	{flac_id,     4, flac_open, flac_close, "flac", N_("FLAC file reader")},
+	{oggflac_id, 32, flac_open, flac_close, "ogg", N_("Ogg FLAC file reader")},
+#endif
 	{NULL, 0, NULL, NULL, NULL, NULL}
 };
 
@@ -57,23 +65,22 @@ input_format *open_audio_file(FILE *in, oe_enc_opt *opt)
 			buf_size = size;
 		}
 
-		if(buf_size > buf_filled)
+		if(size > buf_filled)
 		{
 			ret = fread(buf+buf_filled, 1, buf_size-buf_filled, in);
 			buf_filled += ret;
 
-			if(buf_filled != buf_size)
+			if(buf_filled < size)
 			{ /* File truncated */
-				buf_size = buf_filled;
 				j++;
 				continue;
 			}
 		}
 
-		if(formats[j].id_func(buf, size))
+		if(formats[j].id_func(buf, buf_filled))
 		{
 			/* ok, we now have something that can handle the file */
-			if(formats[j].open_func(in, opt, buf, size)) {
+			if(formats[j].open_func(in, opt, buf, buf_filled)) {
                 free(buf);
 				return &formats[j];
             }
@@ -251,6 +258,8 @@ int aiff_open(FILE *in, oe_enc_opt *opt, unsigned char *buf, int buflen)
 	format.samplesize = READ_U16_BE(buffer+6);
 	format.rate = (int)read_IEEE80(buffer+8);
 
+    aiff->bigendian = 1;
+
 	if(aifc)
 	{
 		if(len < 22)
@@ -258,9 +267,18 @@ int aiff_open(FILE *in, oe_enc_opt *opt, unsigned char *buf, int buflen)
 			fprintf(stderr, _("Warning: AIFF-C header truncated.\n"));
 			return 0;
 		}
-		else if(memcmp(buffer+18, "NONE", 4)) 
+
+		if(!memcmp(buffer+18, "NONE", 4)) 
 		{
-			fprintf(stderr, _("Warning: Can't handle compressed AIFF-C\n"));
+			aiff->bigendian = 1;
+		}
+		else if(!memcmp(buffer+18, "sowt", 4)) 
+		{
+			aiff->bigendian = 0;
+		}
+		else
+		{
+			fprintf(stderr, _("Warning: Can't handle compressed AIFF-C (%c%c%c%c)\n"), *(buffer+18), *(buffer+19), *(buffer+20), *(buffer+21));
 			return 0; /* Compressed. Can't handle */
 		}
 	}
@@ -301,7 +319,6 @@ int aiff_open(FILE *in, oe_enc_opt *opt, unsigned char *buf, int buflen)
 		aiff->channels = format.channels;
 		aiff->samplesize = format.samplesize;
 		aiff->totalsamples = format.totalframes;
-		aiff->bigendian = 1;
 
 		opt->readdata = (void *)aiff;
 
@@ -312,7 +329,7 @@ int aiff_open(FILE *in, oe_enc_opt *opt, unsigned char *buf, int buflen)
 	{
 		fprintf(stderr, 
 				_("Warning: OggEnc does not support this type of AIFF/AIFC file\n"
-				" Must be 8 or 16 bit PCM.\n"));
+				" Must be 8, 16, or 24 bit PCM.\n"));
 		return 0;
 	}
 }
@@ -411,7 +428,10 @@ int wav_open(FILE *in, oe_enc_opt *opt, unsigned char *oldbuf, int buflen)
 
 
 	if( format.align == format.channels*samplesize &&
-			format.samplesize == samplesize*8)
+			format.samplesize == samplesize*8 && 
+    		(format.samplesize == 24 || format.samplesize == 16 || 
+             format.samplesize == 8 ||
+	     (format.samplesize == 32 && format.format == 3)))
 	{
 		/* OK, good - we have the one supported format,
 		   now we want to find the size of the file */
@@ -452,7 +472,7 @@ int wav_open(FILE *in, oe_enc_opt *opt, unsigned char *oldbuf, int buflen)
 	else
 	{
 		fprintf(stderr, 
-				_("ERROR: Wav file is unsupported subformat (must be 16 bit PCM\n"
+				_("ERROR: Wav file is unsupported subformat (must be 8,16, or 24 bit PCM\n"
 				"or floating point PCM\n"));
 		return 0;
 	}
@@ -468,8 +488,9 @@ long wav_read(void *in, float **buffer, int samples)
 	long realsamples;
 
 	if(f->totalsamples && f->samplesread + 
-			bytes_read/(sampbyte*f->channels) > f->totalsamples) 
+			bytes_read/(sampbyte*f->channels) > f->totalsamples) {
 		bytes_read = sampbyte*f->channels*(f->totalsamples - f->samplesread);
+    }
 
 	realsamples = bytes_read/(sampbyte*f->channels);
 	f->samplesread += realsamples;
@@ -485,7 +506,7 @@ long wav_read(void *in, float **buffer, int samples)
 			}
 		}
 	}
-	else
+	else if(f->samplesize==16)
 	{
 		if(!f->bigendian)
 		{
@@ -510,6 +531,32 @@ long wav_read(void *in, float **buffer, int samples)
 			}
 		}
 	}
+    else if(f->samplesize==24) 
+    {
+        if(!f->bigendian) {
+            for(i = 0; i < realsamples; i++)
+            {
+                for(j=0; j < f->channels; j++) 
+                {
+                    buffer[j][i] = ((buf[i*3*f->channels + 3*j + 2] << 16) |
+                      (((unsigned char *)buf)[i*3*f->channels + 3*j + 1] << 8) |
+                      (((unsigned char *)buf)[i*3*f->channels + 3*j] & 0xff)) 
+                        / 8388608.0f;
+
+                }
+            }
+        }
+        else {
+            fprintf(stderr, _("Big endian 24 bit PCM data is not currently "
+                              "supported, aborting.\n"));
+            return 0;
+        }
+    }
+    else {
+        fprintf(stderr, _("Internal error: attempt to read unsupported "
+                          "bitdepth %d\n"), f->samplesize);
+        return 0;
+    }
 
 	return realsamples;
 }
@@ -579,12 +626,11 @@ typedef struct {
     int done;
 } resampler;
 
-long read_resampled(void *d, float **buffer, int samples)
+static long read_resampled(void *d, float **buffer, int samples)
 {
     resampler *rs = d;
     long in_samples;
     int out_samples;
-    int i;
 
     in_samples = res_push_max_input(&rs->resampler, samples);
     if(in_samples > rs->bufsize)
@@ -655,20 +701,63 @@ void clear_resample(oe_enc_opt *opt) {
     free(rs);
 }
 
+typedef struct {
+    audio_read_func real_reader;
+    void *real_readdata;
+    int channels;
+    float scale_factor;
+} scaler;
+
+static long read_scaler(void *data, float **buffer, int samples) {
+    scaler *d = data;
+    long in_samples = d->real_reader(d->real_readdata, buffer, samples);
+    int i,j;
+
+    for(i=0; i < d->channels; i++) {
+        for(j=0; j < in_samples; j++) {
+            buffer[i][j] *= d->scale_factor;
+        }
+    }
+
+    return in_samples;
+}
+
+
+void setup_scaler(oe_enc_opt *opt, float scale) {
+    scaler *d = calloc(1, sizeof(scaler));
+
+    d->real_reader = opt->read_samples;
+    d->real_readdata = opt->readdata;
+
+    opt->read_samples = read_scaler;
+    opt->readdata = d;
+    d->channels = opt->channels;
+    d->scale_factor = scale;
+}
+
+void clear_scaler(oe_enc_opt *opt) {
+    scaler *d = opt->readdata;
+
+    opt->read_samples = d->real_reader;
+    opt->readdata = d->real_readdata;
+
+    free(d);
+}
+
 typedef struct {
     audio_read_func real_reader;
     void *real_readdata;
     float **bufs;
 } downmix;
 
-long read_downmix(void *data, float **buffer, int samples)
+static long read_downmix(void *data, float **buffer, int samples)
 {
     downmix *d = data;
     long in_samples = d->real_reader(d->real_readdata, d->bufs, samples);
     int i;
 
     for(i=0; i < in_samples; i++) {
-        buffer[0][i] = (d->bufs[0][i] + d->bufs[1][i])*0.5;
+        buffer[0][i] = (d->bufs[0][i] + d->bufs[1][i])*0.5f;
     }
 
     return in_samples;
