@@ -3,7 +3,7 @@
  * This program is distributed under the GNU General Public License, version 2.
  * A copy of this license is included with this source.
  *
- * Copyright 2000-2002, Michael Smith <msmith@labyrinth.net.au>
+ * Copyright 2000-2002, Michael Smith <msmith@xiph.org>
  *
  * Portions from Vorbize, (c) Kenneth Arnold <kcarnold@yahoo.com>
  * and libvorbis examples, (c) Monty <monty@xiph.org>
@@ -25,8 +25,8 @@
 #include "i18n.h"
 
 
-#define VERSION_STRING "OggEnc v1.0 (libvorbis 1.0)\n"
-#define COPYRIGHT "(c) 2000-2002 Michael Smith <msmith@labyrinth.net.au>\n"
+#define VERSION_STRING "OggEnc v1.0.1 (libvorbis 1.0.1)\n"
+#define COPYRIGHT "(c) 2000-2003 Michael Smith <msmith@xiph.org>\n"
 
 #define CHUNK 4096 /* We do reads, etc. in multiples of this */
 
@@ -58,7 +58,9 @@ struct option long_options[] = {
     {"managed", 0, 0, 0},
     {"resample",1,0,0},
     {"downmix", 0,0,0},
+    {"scale", 1, 0, 0}, 
     {"advanced-encode-option", 1, 0, 0},
+	{"discard-comments", 0, 0, 0},
 	{NULL,0,0,0}
 };
 	
@@ -75,9 +77,9 @@ int main(int argc, char **argv)
 {
 	/* Default values */
 	oe_options opt = {NULL, 0, NULL, 0, NULL, 0, NULL, 0, NULL, 
-			  0, NULL, 0, NULL, 0, NULL, 0, 0, 0,16,44100,2, 0, NULL,
+			  0, NULL, 0, NULL, 0, NULL, 0, 1, 0, 0,16,44100,2, 0, NULL,
 			  DEFAULT_NAMEFMT_REMOVE, DEFAULT_NAMEFMT_REPLACE, 
-			  NULL, 0, -1,-1,-1,.3,-1,0, 0,0}; 
+			  NULL, 0, -1,-1,-1,.3,-1,0, 0,0.f, 0}; 
 
 	int i;
 
@@ -147,7 +149,9 @@ int main(int argc, char **argv)
         enc_opts.start_encode = start_encode_full;
 		enc_opts.end_encode = final_statistics;
 		enc_opts.error = encode_error;
-		
+		enc_opts.comments = &vc;
+		enc_opts.copy_comments = opt.copy_comments;
+
 		/* OK, let's build the vorbis_comments structure */
 		build_comments(&vc, &opt, i, &artist, &album, &title, &track, 
                 &date, &genre);
@@ -317,12 +321,18 @@ int main(int argc, char **argv)
             else {
                 fprintf(stderr, _("ERROR: Can't downmix except from stereo to mono\n"));
                 errors++;
-                if(opt.resamplefreq)
+                if(opt.resamplefreq && opt.resamplefreq != enc_opts.rate)
                     clear_resample(&enc_opts);
                 goto clear_all;
             }
         }
 
+        if(opt.scale > 0.f) {
+            setup_scaler(&enc_opts, opt.scale);
+            if(!opt.quiet)
+                fprintf(stderr, _("Scaling input to %f\n"), opt.scale);
+        }
+
 
 		if(!enc_opts.total_samples_per_channel)
 			enc_opts.progress_update = update_statistics_notime;
@@ -337,9 +347,11 @@ int main(int argc, char **argv)
 		if(oe_encode(&enc_opts))
 			errors++;
 
+        if(opt.scale > 0)
+            clear_scaler(&enc_opts);
         if(opt.downmix)
             clear_downmix(&enc_opts);
-        if(opt.resamplefreq)
+        if(opt.resamplefreq && opt.resamplefreq != enc_opts.rate)
             clear_resample(&enc_opts);
 clear_all:
 
@@ -395,6 +407,8 @@ static void usage(void)
 		" -s, --serial         Specify a serial number for the stream. If encoding\n"
 		"                      multiple files, this will be incremented for each\n"
 		"                      stream after the first.\n"
+		" --discard-comments   Prevents comments in FLAC and Ogg FLAC files from\n"
+		"                      being copied to the output Ogg Vorbis file.\n"
 		"\n"
 		" Naming:\n"
 		" -o, --output=fn      Write file to fn (only valid in single-file mode)\n"
@@ -411,7 +425,8 @@ static void usage(void)
         "                      Default settings for the above two arguments are platform\n"
         "                      specific.\n"
 		" -c, --comment=c      Add the given string as an extra comment. This may be\n"
-		"                      used multiple times.\n"
+		"                      used multiple times. The argument should be in the\n"
+        "                      format \"tag=value\".\n"
 		" -d, --date           Date for track (usually date of performance)\n"
 		" -N, --tracknum       Track number for this track\n"
 		" -t, --title          Title for this track\n"
@@ -429,14 +444,14 @@ static void usage(void)
 		"                      once, for example, and have it used for all the files)\n"
 		"\n"
 		"INPUT FILES:\n"
-		" OggEnc input files must currently be 16 or 8 bit PCM WAV, AIFF, or AIFF/C\n"
-		" files, or 32 bit IEEE floating point WAV. Files may be mono or stereo\n"
-        " (or more channels) and any sample rate.\n"
+		" OggEnc input files must currently be 24, 16, or 8 bit PCM WAV, AIFF, or AIFF/C\n"
+		" files, 32 bit IEEE floating point WAV, and optionally FLAC or Ogg FLAC. Files\n"
+                "  may be mono or stereo (or more channels) and any sample rate.\n"
 		" Alternatively, the --raw option may be used to use a raw PCM data file, which\n"
-		" must be 16bit stereo little-endian PCM ('headerless wav'), unless additional\n"
+		" must be 16 bit stereo little-endian PCM ('headerless wav'), unless additional\n"
 		" parameters for raw mode are specified.\n"
 		" You can specify taking the file from stdin by using - as the input filename.\n"
-		" In this mode, output is to stdout unless an outfile filename is specified\n"
+		" In this mode, output is to stdout unless an output filename is specified\n"
 		" with -o\n"
 		"\n"), VERSION_STRING, COPYRIGHT);
 }
@@ -556,12 +571,12 @@ static void parse_options(int argc, char **argv, oe_options *opt)
 		{
 			case 0:
                 if(!strcmp(long_options[option_index].name, "managed")) {
-		  if(!opt->managed){
-                    if(!opt->quiet)
-		      fprintf(stderr, 
-			      _("Enabling bitrate management engine\n"));
-                    opt->managed = 1;
-		  }
+		            if(!opt->managed){
+                        if(!opt->quiet)
+            		        fprintf(stderr, 
+                                    _("Enabling bitrate management engine\n"));
+                        opt->managed = 1;
+        		    }
                 }
                 else if(!strcmp(long_options[option_index].name, 
                             "raw-endianness")) {
@@ -590,6 +605,14 @@ static void parse_options(int argc, char **argv, oe_options *opt)
                 else if(!strcmp(long_options[option_index].name, "downmix")) {
                     opt->downmix = 1;
                 }
+                else if(!strcmp(long_options[option_index].name, "scale")) {
+                    opt->scale = atof(optarg);
+				    if(sscanf(optarg, "%f", &opt->scale) != 1) {
+                        opt->scale = 0;
+                        fprintf(stderr, _("Warning: Couldn't parse scaling factor \"%s\"\n"), 
+                                optarg);
+                    }
+                }
                 else if(!strcmp(long_options[option_index].name, "advanced-encode-option")) {
                     char *arg = strdup(optarg);
                     char *val;
@@ -606,6 +629,10 @@ static void parse_options(int argc, char **argv, oe_options *opt)
                     opt->advopt[opt->advopt_count - 1].arg = arg;
                     opt->advopt[opt->advopt_count - 1].val = val;
                 }
+                else if(!strcmp(long_options[option_index].name, "discard-comments")) {
+		  opt->copy_comments = 0;
+		}
+
                 else {
 				    fprintf(stderr, _("Internal error parsing command line options\n"));
 				    exit(1);
