@@ -2,7 +2,7 @@
  *
  * A tool to describe ogg file contents and metadata.
  *
- * Copyright 2002 Michael Smith <msmith@layrinth.net.au>
+ * Copyright 2002 Michael Smith <msmith@xiph.org>
  * Licensed under the GNU GPL, distributed with this program.
  */
 
@@ -23,13 +23,6 @@
 
 #define CHUNK 4500
 
-/* Different implementations have different format strings for 64 bit ints. */
-#ifdef _WIN32
-#define INT64FORMAT "%I64d"
-#else
-#define INT64FORMAT "%Ld"
-#endif
-
 struct vorbis_release {
     char *vendor_string;
     char *desc;
@@ -42,24 +35,23 @@ struct vorbis_release {
         {"Xiphophorus libVorbis I 20011217", "1.0 rc3"},
         {"Xiphophorus libVorbis I 20011231", "1.0 rc3"},
         {"Xiph.Org libVorbis I 20020717", "1.0"},
+        {"Xiph.Org libVorbis I 20030909", "1.0.1"},
         {NULL, NULL},
     };
 
 
 /* TODO:
  *
- * - detect decreasing granulepos
  * - detect violations of muxing constraints
- * - better EOS detection (when EOS not explicitly set)
- * - detect granulepos 'gaps' (possibly vorbis-specific).
- * - check for serial number == (unsigned)-1 (will break some tools?)
- * - more options (e.g. less or more verbose)
+ * - detect granulepos 'gaps' (possibly vorbis-specific). (seperate from 
+ *   serial-number gaps)
  */
 
 typedef struct _stream_processor {
     void (*process_page)(struct _stream_processor *, ogg_page *);
     void (*process_end)(struct _stream_processor *);
     int isillegal;
+    int constraint_violated;
     int shownillegal;
     int isnew;
     long seqno;
@@ -88,8 +80,9 @@ typedef struct {
     vorbis_info vi;
     vorbis_comment vc;
 
-    long bytes;
+    ogg_int64_t bytes;
     ogg_int64_t lastgranulepos;
+    ogg_int64_t firstgranulepos;
 
     int doneheaders;
 } misc_vorbis_info;
@@ -98,6 +91,11 @@ static int printinfo = 1;
 static int printwarn = 1;
 static int verbose = 1;
 
+static int flawed;
+
+#define CONSTRAINT_PAGE_AFTER_EOS   1
+#define CONSTRAINT_MUXING_VIOLATED  2
+
 static stream_set *create_stream_set(void) {
     stream_set *set = calloc(1, sizeof(stream_set));
 
@@ -124,6 +122,7 @@ static void warn(char *format, ...)
 {
     va_list ap;
 
+    flawed = 1;
     if(!printwarn)
         return;
 
@@ -136,6 +135,8 @@ static void error(char *format, ...)
 {
     va_list ap;
 
+    flawed = 1;
+
     va_start(ap, format);
     vfprintf(stdout, format, ap);
     va_end(ap);
@@ -149,6 +150,8 @@ static void vorbis_process(stream_processor *stream, ogg_page *page )
     int k;
 
     ogg_stream_pagein(&stream->os, page);
+    if(inf->doneheaders < 3)
+        header = 1;
 
     while(ogg_stream_packetout(&stream->os, &packet) > 0) {
         if(inf->doneheaders < 3) {
@@ -157,7 +160,6 @@ static void vorbis_process(stream_processor *stream, ogg_page *page )
                        "packet - invalid vorbis stream (%d)\n"), stream->num);
                 continue;
             }
-            header = 1;
             inf->doneheaders++;
             if(inf->doneheaders == 3) {
                 if(ogg_page_granulepos(page) != 0 || ogg_stream_packetpeek(&stream->os, NULL) == 1)
@@ -361,11 +363,20 @@ static void vorbis_process(stream_processor *stream, ogg_page *page )
         ogg_int64_t gp = ogg_page_granulepos(page);
         if(gp > 0) {
             if(gp < inf->lastgranulepos)
-                warn(_("Warning: granulepos in stream %d decreases from " 
-                        INT64FORMAT " to " INT64FORMAT "\n"), stream->num,
-                        inf->lastgranulepos, gp);
+#ifdef _WIN32
+                warn(_("Warning: granulepos in stream %d decreases from %I64d to %I64d" ),
+                        stream->num, inf->lastgranulepos, gp);
+#else
+                warn(_("Warning: granulepos in stream %d decreases from %lld to %lld" ),
+                        stream->num, inf->lastgranulepos, gp);
+#endif
             inf->lastgranulepos = gp;
         }
+        else {
+            warn(_("Negative granulepos on vorbis stream outside of headers. This file was created by a buggy encoder\n"));
+        }
+        if(inf->firstgranulepos < 0) { /* Not set yet */
+        }
         inf->bytes += page->header_len + page->body_len;
     }
 }
@@ -376,16 +387,25 @@ static void vorbis_end(stream_processor *stream)
     long minutes, seconds;
     double bitrate, time;
 
+    /* This should be lastgranulepos - startgranulepos, or something like that*/
     time = (double)inf->lastgranulepos / inf->vi.rate;
     minutes = (long)time / 60;
     seconds = (long)time - minutes*60;
     bitrate = inf->bytes*8 / time / 1000.0;
 
+#ifdef _WIN32
     info(_("Vorbis stream %d:\n"
-           "\tTotal data length: %ld bytes\n"
+           "\tTotal data length: %I64d bytes\n"
            "\tPlayback length: %ldm:%02lds\n"
            "\tAverage bitrate: %f kbps\n"), 
             stream->num,inf->bytes, minutes, seconds, bitrate);
+#else
+    info(_("Vorbis stream %d:\n"
+           "\tTotal data length: %lld bytes\n"
+           "\tPlayback length: %ldm:%02lds\n"
+           "\tAverage bitrate: %f kbps\n"), 
+            stream->num,inf->bytes, minutes, seconds, bitrate);
+#endif
 
     vorbis_comment_clear(&inf->vc);
     vorbis_info_clear(&inf->vi);
@@ -478,6 +498,7 @@ static stream_processor *find_stream_processor(stream_set *set, ogg_page *page)
     ogg_uint32_t serial = ogg_page_serialno(page);
     int i, found = 0;
     int invalid = 0;
+    int constraint = 0;
     stream_processor *stream;
 
     for(i=0; i < set->used; i++) {
@@ -490,6 +511,7 @@ static stream_processor *find_stream_processor(stream_set *set, ogg_page *page)
             /* if we have detected EOS, then this can't occur here. */
             if(stream->end) {
                 stream->isillegal = 1;
+                stream->constraint_violated = CONSTRAINT_PAGE_AFTER_EOS;
                 return stream;
             }
 
@@ -506,8 +528,10 @@ static stream_processor *find_stream_processor(stream_set *set, ogg_page *page)
      * XXX: might this sometimes catch ok streams if EOS flag is missing,
      * but the stream is otherwise ok?
      */
-    if(streams_open(set) && !set->in_headers)
+    if(streams_open(set) && !set->in_headers) {
+        constraint = CONSTRAINT_MUXING_VIOLATED;
         invalid = 1;
+    }
 
     set->in_headers = 1;
 
@@ -524,6 +548,7 @@ static stream_processor *find_stream_processor(stream_set *set, ogg_page *page)
 
     stream->isnew = 1;
     stream->isillegal = invalid;
+    stream->constraint_violated = constraint;
 
     {
         int res;
@@ -559,6 +584,11 @@ static stream_processor *find_stream_processor(stream_set *set, ogg_page *page)
    stream->end = ogg_page_eos(page);
    stream->serial = serial;
 
+   if(stream->serial == 0 || stream->serial == -1) {
+       info(_("Note: Stream %d has serial number %d, which is legal but may "
+              "cause problems with some tools."), stream->num, stream->serial);
+   }
+
    return stream;
 }
 
@@ -571,8 +601,11 @@ static int get_next_page(FILE *f, ogg_sync_state *sync, ogg_page *page,
 
     while((ret = ogg_sync_pageout(sync, page)) <= 0) {
         if(ret < 0)
-            warn(_("Warning: Hole in data found at approximate offset "
-                   INT64FORMAT " bytes. Corrupted ogg.\n"), *written);
+#ifdef _WIN32
+            warn(_("Warning: Hole in data found at approximate offset %I64d bytes. Corrupted ogg.\n"), *written);
+#else
+            warn(_("Warning: Hole in data found at approximate offset %lld bytes. Corrupted ogg.\n"), *written);
+#endif
 
         buffer = ogg_sync_buffer(sync, CHUNK);
         bytes = fread(buffer, 1, CHUNK, f);
@@ -615,10 +648,28 @@ static void process_file(char *filename) {
         }
 
         if(p->isillegal && !p->shownillegal) {
+            char *constraint;
+            switch(p->constraint_violated) {
+                case CONSTRAINT_PAGE_AFTER_EOS:
+                    constraint = _("Page found for stream after EOS flag");
+                    break;
+                case CONSTRAINT_MUXING_VIOLATED:
+                    constraint = _("Ogg muxing constraints violated, new "
+                                   "stream before EOS of all previous streams");
+                    break;
+                default:
+                    constraint = _("Error unknown.");
+            }
+
             warn(_("Warning: illegally placed page(s) for logical stream %d\n"
-                   "This indicates a corrupt ogg file.\n"), p->num);
+                   "This indicates a corrupt ogg file: %s.\n"), 
+                    p->num, constraint);
             p->shownillegal = 1;
-            continue;
+            /* If it's a new stream, we want to continue processing this page
+             * anyway to suppress additional spurious errors
+             */
+            if(!p->isnew)
+                continue;
         }
 
         if(p->isnew) {
@@ -651,6 +702,7 @@ static void process_file(char *filename) {
                     p->process_end(p);
                 info(_("Logical stream %d ended\n"), p->num);
                 p->isillegal = 1;
+                p->constraint_violated = CONSTRAINT_PAGE_AFTER_EOS;
             }
         }
     }
@@ -668,9 +720,9 @@ static void process_file(char *filename) {
 
 static void usage(void) {
     printf(_("ogginfo 1.0\n"
-             "(c) 2002 Michael Smith <msmith@labyrinth.net.au>\n"
+             "(c) 2002 Michael Smith <msmith@xiph.org>\n"
              "\n"
-             "Usage: ogginfo [flags] files1.ogg [file2.ogg ... fileN.ogg]\n"
+             "Usage: ogginfo [flags] file1.ogg [file2.ogg ... fileN.ogg]\n"
              "Flags supported:\n"
              "\t-h Show this help message\n"
              "\t-q Make less verbose. Once will remove detailed informative\n"
@@ -721,10 +773,15 @@ int main(int argc, char **argv) {
         return 1;
     }
 
+    ret = 0;
+
     for(f=optind; f < argc; f++) {
+        flawed = 0;
         process_file(argv[f]);
+        if(flawed != 0)
+            ret = flawed;
     }
 
-    return 0;
+    return ret;
 }
 
